from typing import Optional

from whois import whois, parser
from aiogram.types import Message
from dataclasses import dataclass
from aiogram.utils.markdown import quote_html

from tgbot.handlers.base import SimpleCommandHandler
from tgbot.handlers.errors import NotEnoughArgs, LocalhostForbidden
from tgbot.middlewares.throttling import rate_limit

whois_help_message = """
❓ Вернёт информацию о домене.

 Использование: `/whois <домен>`
"""

no_domain_text = """
❗Не указан домен или указан неверный/несуществующий домен.

Напишите /whois чтобы посмотреть справку.
"""


@dataclass
class DomainAttrClass:
    icon: str
    name: str
    attr: str


# DOMAIN_ATTR_CLASSES order have matter!
DOMAIN_ATTR_CLASSES = [
    DomainAttrClass("👤", "Регистратор", "registrar"),
    DomainAttrClass("📅", "Дата создания", "creation_date"),
    DomainAttrClass("📅", "Дата окончания", "expiration_date"),
    DomainAttrClass("📖", "Адрес", "address"),
    DomainAttrClass("🏘", "Город", "city"),
    DomainAttrClass("🏘", "Страна", "country"),
    DomainAttrClass("💬", "Имя", "name"),
    DomainAttrClass("💼", "Организация", "org"),
    DomainAttrClass("💬", "Zipcode", "zipcode"),
    DomainAttrClass("✉", "Почта", "emails"),
    DomainAttrClass("📌", "NS", "name_servers"),
    DomainAttrClass("🔐", "DNSSec", "dnssec"),
]


def create_whois_message(domain: str) -> str:
    try:
        domain_info = whois(domain)
    except parser.PywhoisError as e:
        return f"❗ Домен {domain} свободен или не был найден."
    domain_name = domain_info.get("domain_name")
    if domain_name is None:
        return no_domain_text
    if isinstance(domain_name, list):
        domain_name = domain_name[0]

    message = f"\n📝 Информация о домене {domain_name.lower()}:"

    for i, domain_attr in enumerate(DOMAIN_ATTR_CLASSES):
        # for pretty printing, DOMAIN_ATTR_CLASSES order have matter!
        if i in [2, 10]:
            message += "\n"
        resp = format_domain_item(
            domain_attr.icon, domain_attr.name, domain_info.get(domain_attr.attr)
        )
        if resp:
            message += resp

    return message


def format_domain_item(icon, item_name, items) -> Optional[str]:
    if not items:
        return
    if isinstance(items, list):
        message = f"\n{icon} {item_name}:\n"
        message += str.join("\n", [f" * <code>{ns}</code>" for ns in list(set(map(str.lower, items)))])
    else:
        message = f"\n{icon} {item_name}: {items}"
    return message


class WhoisCommandHandler(SimpleCommandHandler):
    help_message = whois_help_message

    def __init__(self):
        super().__init__()

    @rate_limit
    async def handler(self, message: Message):
        try:
            args = await self.process_args(message.text)
        except NotEnoughArgs:
            await message.answer(no_domain_text, parse_mode='Markdown')
        except LocalhostForbidden:
            await message.answer(self.localhost_forbidden_message, parse_mode='Markdown')
        else:
            await message.answer(create_whois_message(args[0]), parse_mode='html')

    async def process_args(self, text: str) -> list:
        args = text.split()
        if len(args) == 1:
            raise NotEnoughArgs
        if len(args) >= 2:
            host = args[1]
            await self.validate_target(host)
            return [host]  # only domain name

    async def prepare_message(self) -> str:
        pass
